%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% ################################################################# %% DESCRIPTION
% analysePCELLS  overall analysis function
% This function can be used with the preprocessed dataset available to download from:
%
% It will then load the necessary files and generate figures based on the data saved within them
% Along with the figures it will also perform statistical tests. These analyses, test results and
% figures are all identical to those reported in the main text. Where permutations or shuffles are
% performed be aware that the results will vary on iteration.
%
% USAGE:
%         run the function with sdir set to the folder containing the data files
%
% INPUT:
%         no inputs necessary
%
% OUTPUT:
%    no outputs, but figures will be saved and results displayed at the command line
%
% See also: 

% HISTORY:
% version 1.0.0, Release 30/09/19 Initial release
%
% Author: Roddy Grieves
% UCL, 26 Bedford Way
% eMail: r.grieves@ucl.ac.uk
% Copyright 2019 Roddy Grieves

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% ################################################################# %% Initial variables
%     sdir = 'Y:\Jeffery lab\Roddy G\Experiments\Lattice Maze\Data\klusdata'; % starting directory, where figures will be saved and data files are stored
    sdir = 'Y:\Jeffery lab\Roddy G\Experiments\Lattice Maze\Data\klusdata\Summary data';
    cd(sdir); % move there
    close all; % close any competing figures

    snames = {'square1','lattice','square2'}; % the part names as they are listed in the data files, aligned and tilted are called lattice at this level
    snames3 = {'square1_3d','lattice_3d','square2_3d'}; % 3D versions of the above, both are used in different places

%% if tdata doesn't exist in memory (the largest and most important variable) load everything
    if ~exist('tdata','var')
        load([sdir '\tabulatedDATA.mat'])
        load([sdir '\tabulatedDATAbehave.mat'])
        load([sdir '\tabulatedDATAdcdfs.mat'])
        load([sdir '\tabulatedDATAdwell.mat'])
        load([sdir '\tabulatedDATAfields.mat'])
        load([sdir '\tabulatedDATAslice.mat'])
        load([sdir '\tabulatedDATAexfdata.mat'])
    end

%% colours for plotting
    c = bone(4);
    colsq = c(3,:);
    collt = c(2,:);
    coldi = c(1,:);

    colx = rgb('Silver');
    coly = rgb('DimGray');
    colz = rgb('Black');
    cola = rgb('PaleTurquoise');
    colb = rgb('SteelBlue');
    colc = rgb('MidnightBlue');

%% names of the mazes for plotting
    m1name = 'Arena';
    m2name = 'Aligned';
    m3name = 'Tilted';

%% Settings for filtering cells to get place cells. Note that most of these are not used and instead we rely on tdata.cell_type which is the manual
% cell type classification, which in turn was based on automatic cell classification based on many of these parameters
% They are only maintained here for exploratory analysis
    SI_thresh = 0; % threshold of spatial information to consider a cell a place cell
    WW_thresh = 250; % threshold of width of waveform to consider a cell a place cell
    FR_thresh = [0.1 10]; % threshold of firing rate to consider a cell a place cell
    SP_thresh = 100; % sparsity threshold
    active_frate = 1; % firing rate a cell has to have to be considered 'active'
    fv_thresh = 5; % min number of place field visits
    pcut = 20; % min place cells in decoding
    fvol_thresh = [64 inf];

%% %%%%%%%%%% NatComms reviewed figs with meaningful names
% the function will run through this structure and run any functions set to true or 1
    % Main figures
    config = struct;
    config.FIG1_maze_schematics_and_photos                    = 0; % Fig. 1
    config.FIG2_behaviour_dwell_time                          = 0; % Fig. 2
    config.FIG3_main_example_cells                            = 0; % Fig. 3
    config.FIG4_field_per_cell_and_volume                     = 0; % Fig. 4
    config.FIG5_field_centroids                               = 0; % Fig. 5
    config.FIG6_axis_lengths_elongation                       = 0; % Fig. 6
    config.FIG7_field_orientation                             = 1; % Fig. 7
    config.FIG8_spatial_info_field_extents                    = 0; % Fig. 8
    
    % Supplementary figures
    config.FIGS2_eigen_schematic                              = 0; % Fig. S2 (S1 in powerpoint)
    config.FIGS3_binmorph_schematic                           = 0; % Fig. S3
    config.FIGS4_autocorr_schematic                           = 0; % Fig. S4
    config.FIGS5_coverage_speed                               = 0; % Fig. S5
    config.FIGS6_aligned_examples                             = 0; % Fig. S6
    config.FIGS7_tilted_examples                              = 0; % Fig. S7
    config.FIGS8_recording_stability                          = 0; % Fig. S8 (S6 & S7 in powerpoint)
    config.FIGS9_zingg_results                                = 0; % Fig. S9
    config.FIGS10_downsampling                                = 0; % Fig. S10
    config.FIGS11_axis_lengths_elongation                     = 0; % Fig. S11
    config.FIGS12_alternative_orientation                     = 0; % Fig. S12
    config.FIGS13_spatial_info_field_extents                  = 1; % Fig. S13
    config.FIGS14_relation_between_mazes                      = 0; % Fig. S14
    config.FIGS15_field_stability                             = 0; % Fig. S15
    config.FIGS16_theta_speed                                 = 0; % Fig. S16
    config.FIGS17_cells_speed                                 = 0; % Fig. S17
    config.FIGS18_cells_theta_pprecess                        = 0; % Fig. S18 (S19 in powerpoint)
    
%% Figure settings
    plotfigures = 1; % set to 1 to make the figures and display stats
    print_now = 1; % should the function print the figures? If 1 the figures will be set to invisible
    print_qual = 2000; % quality for printing in DPI
    load_now = 1; % some functions have to load real data, this can be disabled for time
    plotall = 1; % should the function plot everything, including examples etc?
    fsiz = 8; % the fontsize for different texts
    flnw = 0.5; % the line width for different plots

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% ################################################################# %% Generate indices
%% Generate summary statistics
    % get vectors that will be useful for arranging data
    rindx = tdata.rat(:);
    pindx = tdata.part(:);

    lindx = tdata.lat_type(:);
    pindxf = fdata.part(:);
    pindxb = bdata.part(:);
    cindxM = tdata.cell_type(:);
    cindxMf = fdata.cell_type(:);
    sindx = tdata.spatial_information(:);
    fme = tdata.field_mean_eigenvector(:,:); % field mean eigenvectors
    far = tdata.field_total_area_ratios(:,:); % field area ratios

%% For each data file there are four logical vectors which can be used to index into them
% These give the rows for place cells in each maze, where indices ending in sq1 correspond to the first arena
% sq2 the second arena, la1 the aligned lattice and la2 the tilted lattice
% Note that throughout the functions the arena may be called the square and the tilted lattice may be called
% the diagonal lattice (old names)
    % Get an index of all place cells in each maze
    pc_idx = cindxM==1 & tdata.firing_rate_Hz>FR_thresh(1) & tdata.firing_rate_Hz<FR_thresh(2) & tdata.sparsity<SP_thresh & tdata.spatial_information>SI_thresh;
    pc_idx_sq1 = cindxM==1 & tdata.firing_rate_Hz>FR_thresh(1) & tdata.firing_rate_Hz<FR_thresh(2) & tdata.sparsity<SP_thresh & tdata.spatial_information>SI_thresh & tdata.part==1;
    pc_idx_sq2 = cindxM==1 & tdata.firing_rate_Hz>FR_thresh(1) & tdata.firing_rate_Hz<FR_thresh(2) & tdata.sparsity<SP_thresh & tdata.spatial_information>SI_thresh & tdata.part==3;
    pc_idx_la1 = cindxM==1 & tdata.firing_rate_Hz>FR_thresh(1) & tdata.firing_rate_Hz<FR_thresh(2) & tdata.sparsity<SP_thresh & tdata.spatial_information>SI_thresh & tdata.part==2 & tdata.lat_type==1;
    pc_idx_la2 = cindxM==1 & tdata.firing_rate_Hz>FR_thresh(1) & tdata.firing_rate_Hz<FR_thresh(2) & tdata.sparsity<SP_thresh & tdata.spatial_information>SI_thresh & tdata.part==2 & tdata.lat_type==2;

    % Get in index of sessions in each maze
    pb_idx_sq1 = bdata.part(:)==1;
    pb_idx_sq2 = bdata.part(:)==3;
    pb_idx_la1 = bdata.part(:)==2 & bdata.lat_type(:)==1;
    pb_idx_la2 = bdata.part(:)==2 & bdata.lat_type(:)==2;

    % Get an index of place fields in each maze
    pf_idx_sq1 = ismember(fdata.uci,tdata.uci(pc_idx_sq1)) & fdata.part(:)==1 & fdata.field_voxel_volume(:)>fvol_thresh(1) & fdata.field_voxel_volume(:)<fvol_thresh(2) & fdata.field_num_visits(:)>fv_thresh;
    pf_idx_sq2 = ismember(fdata.uci,tdata.uci(pc_idx_sq2)) & fdata.part(:)==3 & fdata.field_voxel_volume(:)>fvol_thresh(1) & fdata.field_voxel_volume(:)<fvol_thresh(2) & fdata.field_num_visits(:)>fv_thresh;
    pf_idx_la1 = ismember(fdata.uci,tdata.uci(pc_idx_la1)) & fdata.part(:)==2 & fdata.lat_type(:)==1 & fdata.field_voxel_volume(:)>fvol_thresh(1) & fdata.field_voxel_volume(:)<fvol_thresh(2) & fdata.field_num_visits(:)>fv_thresh;
    pf_idx_la2 = ismember(fdata.uci,tdata.uci(pc_idx_la2)) & fdata.part(:)==2 & fdata.lat_type(:)==2 & fdata.field_voxel_volume(:)>fvol_thresh(1) & fdata.field_voxel_volume(:)<fvol_thresh(2) & fdata.field_num_visits(:)>fv_thresh;

    % Get an index of slice data in each maze
    sl_idx_sq1 = ismember(sldata.uci,tdata.uci(pc_idx_sq1)) & sldata.part(:)==1;
    sl_idx_sq2 = ismember(sldata.uci,tdata.uci(pc_idx_sq2)) & sldata.part(:)==3;
    sl_idx_la1 = ismember(sldata.uci,tdata.uci(pc_idx_la1)) & sldata.part(:)==2 & sldata.lat_type==1;
    sl_idx_la2 = ismember(sldata.uci,tdata.uci(pc_idx_la2)) & sldata.part(:)==2 & sldata.lat_type==2;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% ################################################################# %% Generate figures
%% Run through the config structure and generate all requested figures
    if print_now
        fig_vis = 'off';
    else
        fig_vis = 'on';
    end
    if plotfigures
        fnames = fieldnames(config);
        for ff = 1:length(fnames)
            val = config.(fnames{ff});
            if val
                disp(sprintf('\trunning: %s',fnames{ff}))
                eval([fnames{ff} ';']);
            end
        end
    end
    
    






