%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% ################################################################# %% Created for
%
%     The place-cell representation of volumetric space in rats
%     Roddy M. Grieves, Selim Jedidi-Ayoub, Karyna Mishchanchuk, Anyi Liu, Sophie Renaudineau and Kate J. Jeffery
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% ################################################################# %% DESCRIPTION
% FIGS15_field_stability small function to make Fig. S15
% Figure S15 shows an analysis to determine if cells were stable within sessions
%
% USAGE:
%         Should be run via analysePCELLS
%
% INPUT:
%         No inputs, this is a script that relies on analysePCELLS to load data
%
% OUTPUT:
%    Will save a figure if print_now is set to 1 in analysePCELLS
%
% See also: analysePCELLS

% HISTORY:
% version 1.0.0, Release 30/09/19 Initial release
%
% Author: Roddy Grieves
% UCL, 26 Bedford Way
% eMail: r.grieves@ucl.ac.uk
% Copyright 2019 Roddy Grieves

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% ################################################################# %% Initial settings
close all
disp(sprintf('Running paper_3_stability...'))
pnames = {'arena 1','aligned','tilted','tilted'};

if ~exist('stabmat','var')
    load('stabDATA.mat')
end
stab_sq1_idx = stabmat.part(:)==1;
stab_sq2_idx = stabmat.part(:)==3;
stab_la1_idx = stabmat.part(:)==2 & stabmat.lat_type==1;
stab_la2_idx = stabmat.part(:)==2 & stabmat.lat_type==2;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% ################################################################# %% Compare first and second halves of each maze dimension projection to test field stability
disp(sprintf('\tComparing first and second half for every projection'))
    index_list = {stab_sq1_idx,stab_la1_idx,stab_la2_idx,stab_la2_idx};
    obs_field_stab = NaN(2000,4,length(index_list));
    all_field_maps = cell(2000,2,3);

    for pp = 1:4 % for each part/maze
        disp(sprintf('\t\t...maze: %s',pnames{pp}))        
        current_index = index_list{pp};
        cell_indx = unique(stabmat.uci(current_index));

        loopout = looper(length(cell_indx));
        for cc = 1:length(cell_indx) % for every cell
            cidx = stabmat.uci(:)==cell_indx(cc); % index for current cell

            for dd = 1:4 % for each dimension/projection
                if pp>3
                    dim_list = {'ab','ac','bc','abc'};
                else
                    dim_list = {'xy','xz','yz','xyz'};                    
                end
                    
                % skip this cell if it is missing the part we want
                if isempty(stabmat.uci(cidx & current_index))
                    loopout = looper(loopout);
                    continue
                end

                % get both projected half maps
                map1 = stabmat.(['map_' dim_list{dd} '_h1']){cidx & current_index};
                map2 = stabmat.(['map_' dim_list{dd} '_h2']){cidx & current_index}; 

                % if one of the maps are missing skip this cell
                if any([isempty(map1) isempty(map2)])
                    loopout = looper(loopout);        
                    continue
                end

                % if both maps have a firing rate less than 1Hz skip this cell
                if all([nanmax(map1(:))<1 nanmax(map2(:))<1])
                    loopout = looper(loopout);        
                    continue
                end
                all_field_maps{cc,1,dd,pp} = map1;
                all_field_maps{cc,2,dd,pp} = map2;                
                
                % compute correlation
                [r,p] = corr(map1(:),map2(:),'Type','Pearson','Rows','pairwise');
                obs_field_stab(cc,dd,pp) = r;             
                loopout = looper(loopout);
            end   
        end
    end

%% ##################################### Compare first and second halves of each maze dimension projection randomly to test field stability
disp(sprintf('\tComparing first and second half for every projection (shuffled)'))
    shuff = 1000;
    shuff_field_stab = NaN(shuff,4,length(index_list));
    ncells = size(all_field_maps,1);

    for pp = 1:4 % for each part/maze
        disp(sprintf('\t\t...maze: %s',pnames{pp}))        

        loopout = looper(shuff);
        for cc = 1:shuff % for every shuffle

            for dd = 1:4 % for every projection dimension
                shuff_wait = true;
                while shuff_wait
                    rindx = randperm(ncells,2); % get a random cell index for 2 cells
                    map1 = all_field_maps{rindx(1),1,dd,pp}; % get first half map for this projection {cell,half,dim}
                    map2 = all_field_maps{rindx(2),2,dd,pp}; % get second half map for this projection {cell,half,dim}
                    if any([isempty(map1) isempty(map2)]) | all([nanmax(map1(:))<1 nanmax(map2(:))<1]) % if either map is missing or if both maps are <1Hz rerun while loop
                        continue
                    end 
                    [r,p] = corr(map1(:),map2(:),'Type','Pearson','Rows','pairwise'); % otherwise correlate the maps
                    shuff_field_stab(cc,dd,pp) = r; % save the r value
                    shuff_wait = false; % stop this while loop
                end
            end

            loopout = looper(loopout);
        end
    end
 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% ################################################################# %% FUNCTION BODY
    fignow = figure('visible',fig_vis,'Position',[50,60,595,842]);
    set(gcf,'InvertHardCopy','off'); % this stops white lines being plotted black      
    set(gcf,'color','w'); % makes the background colour white
    colormap(jet(256)); % to make sure the colormap is not the horrible default one

    annotation('textbox','Units','pixels','Position',[1, 830, 300, 20],'string','a','FontSize',18,'LineStyle','none','interpreter','none'); 
    annotation('textbox','Units','pixels','Position',[1, 700, 300, 20],'string','b','FontSize',18,'LineStyle','none','interpreter','none');     
    
    annotation('textbox','Units','pixels','Position',[10, 670, 300, 20],'string',sprintf('Arena\nprojected'),'FontSize',fsiz+2,'LineStyle','none','interpreter','none'); 
    annotation('textbox','Units','pixels','Position',[10, 505, 300, 20],'string',sprintf('Aligned\nprojected'),'FontSize',fsiz+2,'LineStyle','none','interpreter','none'); 
    annotation('textbox','Units','pixels','Position',[10, 340, 300, 20],'string',sprintf('Tilted\nprojected'),'FontSize',fsiz+2,'LineStyle','none','interpreter','none'); 
    
    annotation('textbox','Units','pixels','Position',[400, 670, 300, 20],'string',sprintf('Arena\nvolume'),'FontSize',fsiz+2,'LineStyle','none','interpreter','none'); 
    annotation('textbox','Units','pixels','Position',[400, 505, 300, 20],'string',sprintf('Aligned\nvolume'),'FontSize',fsiz+2,'LineStyle','none','interpreter','none'); 
    annotation('textbox','Units','pixels','Position',[400, 340, 300, 20],'string',sprintf('Tilted\nvolume'),'FontSize',fsiz+2,'LineStyle','none','interpreter','none'); 

% %% ##################################### example 1st and second halves
%     dirs = {'Y:\Jeffery lab\Roddy G\Experiments\Lattice Maze\Data\Place cells\852_P3_16_HPC\170320_kk_kt_ll_k3\klustest3\kwiktint',7,2,852,170320;...
%             'Y:\Jeffery lab\Roddy G\Experiments\Lattice Maze\Data\Place cells\852_P3_16_HPC\170320_kk_kt_ll_k3\klustest3\kwiktint',7,2,852,170320;...
%             'Y:\Jeffery lab\Roddy G\Experiments\Lattice Maze\Data\Place cells\852_P3_16_HPC\170725_kk_kt_ll_k3\klustest3\kwiktint',7,1,852,170725;...
%             'Y:\Jeffery lab\Roddy G\Experiments\Lattice Maze\Data\Place cells\852_P3_16_HPC\170725_kk_kt_ll_k3\klustest3\kwiktint',7,1,852,170725};    
    dirs = {[sdir '\852_20032017.mat'],7,2,852,170320;...
            [sdir '\852_20032017.mat'],7,2,852,170320;...
            [sdir '\852_25072017.mat'],7,1,852,170725;...
            [sdir '\852_25072017.mat'],7,1,852,170725};    

    annotation('textbox','Units','pixels','Position',[100, 818, 300, 20],'string',sprintf('%s %s %s %s',num2str(dirs{1,4}),num2str(dirs{1,5}),num2str(dirs{1,2}),num2str(dirs{1,3})),'FontSize',fsiz,'LineStyle','none','interpreter','none'); 
    annotation('textbox','Units','pixels','Position',[350, 818, 300, 20],'string',sprintf('%s %s %s %s',num2str(dirs{3,4}),num2str(dirs{3,5}),num2str(dirs{3,2}),num2str(dirs{3,3})),'FontSize',fsiz,'LineStyle','none','interpreter','none'); 
                
    axpos = {[30 700 150 150],[130 700 150 150],[260 700 150 150],[360 700 150 150]};
    for dd = 1:size(dirs,1)   
        disp(sprintf('\t...cell: %d',dd));

        % collect the data we want
        if isodd(dd)
            cd(sdir);
            load(dirs{dd,1},'-mat','sdata');             
            uci = ['r' sdata.rat_num '_' sdata.date '_t' num2str(dirs{dd,2}) '_c' num2str(dirs{dd,3})]; 
            ratemap = sdata.(uci).lattice_3d.ratemap_half1;
        elseif iseven(dd)
            ratemap = sdata.(uci).lattice_3d.ratemap_half2;
        end
        lx2 = sdata.lattice_3d.lxrm;
        ly2 = sdata.lattice_3d.lyrm;
        lz2 = sdata.lattice_3d.lzrm;       

        % plot data
        ax_stab1 = axes('Units','pixels','Position',axpos{dd});
            ratemap(ratemap < 0.1*nanmax(ratemap(:))) = NaN;
            vol3d('cdata',ratemap,'texture','3D');
            alphamap('rampup');
            colormap(gca,'jet');
            caxis([0 nanmax(ratemap(:))]);

            hold on
            edg = [max(lx2)-min(lx2), max(ly2)-min(ly2), max(lz2)-min(lz2)];
            ori = [min(lx2), min(ly2), min(lz2)]; 
            plotcube(edg,ori,0,0,0.3);

            axis off
            pad_valr = 6;
            axis([min(lx2)-pad_valr,max(lx2)+pad_valr,min(ly2)-pad_valr,max(ly2)+pad_valr,min(lz2)-pad_valr,max(lz2)+pad_valr]);        
            view(30,22)
            daspect([1 1 1])
            camproj perspective
            rotate3d on 
    end    
    cd(sdir);

%% colorbar for 3D ratemap
    axc_cmap = axes('Units','pixels','Position',[axpos{end}(1)+160 axpos{end}(2)+40 10 80]);
        cmap = repmat(reshape(1:100,[],1),1,1,2);
        cmap(cmap < 0.2*nanmax(cmap(:))) = NaN;
        vol3d('cdata',cmap,'texture','3D');
        alphamap('rampup');
        colormap(gca,'jet');
        caxis([0.2*nanmax(cmap(:)) nanmax(cmap(:))]);
        axis xy on tight
        box on
        ax = gca;
        ax.XTick = [];
        ax.YTick = [20];
        ax.YTickLabel = {'10%'};
        text(0.5,-0.08,'0 Hz','FontSize',fsiz,'Units','normalized','HorizontalAlignment','center','Color','k');
        text(0.5,1.08,'Max','FontSize',fsiz,'Units','normalized','HorizontalAlignment','center','Color','k');
    
    
%% ##################################### Schematics of projections   
xs = [80 140 260];
spacings = 120;
alp = .2;

%% Arena
% graphic slice Z
    ax_Btop = axes('Units','pixels','Position',[xs(1) 635 60 60]);
        xpatch = ([0 1 1 0 0 0; 1 1 0 0 1 1; 1 1 0 0 1 1; 0 1 1 0 0 0]);
        ypatch = ([0 0 1 1 0 0; 0 1 1 0 0 0; 0 1 1 0 1 1; 0 0 1 1 1 1]);
        zpatch = ([0 0 0 0 0 1; 0 0 0 0 0 1; 1 1 1 1 0 1; 1 1 1 1 0 1]).*0.2;
        for vv = 1:6
            h = patch(xpatch(:,vv),ypatch(:,vv),zpatch(:,vv),'w');
            set(h,'edgecolor','k','FaceColor','none')
        end      
        hold on
        surf([0 0; 1 1],[0 1; 0 1],[0 0; 0 0].*2,'FaceColor','b','FaceAlpha',alp); axis off; view(30,22); daspect([1 1 1]); axis vis3d; camproj perspective; rotate3d on    
        
% graphic slice X
    ax_Bmid = axes('Units','pixels','Position',ax_Btop.Position+[spacings 0 0 0]);
        for vv = 1:6
            h = patch(xpatch(:,vv),ypatch(:,vv),zpatch(:,vv),'w');
            set(h,'edgecolor','k','FaceColor','none')
        end         
        hold on;
        surf([0 0;0 0],[0 0; 1 1],[0 1; 0 1].*.2,'FaceColor','b','FaceAlpha',alp); axis off; view(30,22); daspect([1 1 1]); axis vis3d; camproj perspective; rotate3d on   
        
% graphic slice Y
    ax_Bbot = axes('Units','pixels','Position',ax_Bmid.Position+[spacings 0 0 0]);
        for vv = 1:6
            h = patch(xpatch(:,vv),ypatch(:,vv),zpatch(:,vv),'w');
            set(h,'edgecolor','k','FaceColor','none','FaceAlpha',.5)
        end        
        hold on
        surf([1 0; 1 0],[1 1;1 1],[0 0; 1 1].*.2,'FaceColor','b','FaceAlpha',alp); axis off; view(30,22); daspect([1 1 1]); axis vis3d; camproj perspective; rotate3d on  
        
% graphic XYZ volume
    ax_Bmid = axes('Units','pixels','Position',ax_Bbot.Position+[170 0 0 0]);
        for vv = 1:6
            h = patch(xpatch(:,vv),ypatch(:,vv),zpatch(:,vv),'b');
            set(h,'edgecolor','k','FaceColor','b','FaceAlpha',alp/2)
        end   
        axis off; view(30,22); daspect([1 1 1]); axis vis3d; camproj perspective; rotate3d on   
        
%% Aligned
% graphic slice Z
    ax_Btop = axes('Units','pixels','Position',[xs(1) 475 60 60]);
        xpatch = ([0 1 1 0 0 0; 1 1 0 0 1 1; 1 1 0 0 1 1; 0 1 1 0 0 0]);
        ypatch = ([0 0 1 1 0 0; 0 1 1 0 0 0; 0 1 1 0 1 1; 0 0 1 1 1 1]);
        zpatch = ([0 0 0 0 0 1; 0 0 0 0 0 1; 1 1 1 1 0 1; 1 1 1 1 0 1]);
        for vv = 1:6
            h = patch(xpatch(:,vv),ypatch(:,vv),zpatch(:,vv),'w');
            set(h,'edgecolor','k','FaceColor','none')
        end      
        hold on
        surf([0 0; 1 1],[0 1; 0 1],[0 0; 0 0],'FaceColor','b','FaceAlpha',alp); axis off; view(30,22); daspect([1 1 1]); axis vis3d; camproj perspective; rotate3d on    
        
% graphic slice X
    ax_Bmid = axes('Units','pixels','Position',ax_Btop.Position+[spacings 0 0 0]);
        for vv = 1:6
            h = patch(xpatch(:,vv),ypatch(:,vv),zpatch(:,vv),'w');
            set(h,'edgecolor','k','FaceColor','none')
        end         
        hold on;
        surf([0 0;0 0],[0 0; 1 1],[0 1; 0 1],'FaceColor','b','FaceAlpha',alp); axis off; view(30,22); daspect([1 1 1]); axis vis3d; camproj perspective; rotate3d on 
        
% graphic slice Y
    ax_Bbot = axes('Units','pixels','Position',ax_Bmid.Position+[spacings 0 0 0]);
        for vv = 1:6
            h = patch(xpatch(:,vv),ypatch(:,vv),zpatch(:,vv),'w');
            set(h,'edgecolor','k','FaceColor','none')
        end        
        hold on
        surf([1 0; 1 0],[1 1;1 1],[0 0; 1 1],'FaceColor','b','FaceAlpha',alp); axis off; view(30,22); daspect([1 1 1]); axis vis3d; camproj perspective; rotate3d on   
        
% graphic XYZ volume
    ax_Bmid = axes('Units','pixels','Position',ax_Bbot.Position+[170 0 0 0]);
        for vv = 1:6
            h = patch(xpatch(:,vv),ypatch(:,vv),zpatch(:,vv),'b');
            set(h,'edgecolor','k','FaceColor','b','FaceAlpha',alp/2)
        end   
        axis off; view(30,22); daspect([1 1 1]); axis vis3d; camproj perspective; rotate3d on 
        
        
%% Tilted
% graphic slice Z
    ax_Btop = axes('Units','pixels','Position',[xs(1)-10 300-5 90 90]);
        xpatch = ([0 1 1 0 0 0; 1 1 0 0 1 1; 1 1 0 0 1 1; 0 1 1 0 0 0]);
        ypatch = ([0 0 1 1 0 0; 0 1 1 0 0 0; 0 1 1 0 1 1; 0 0 1 1 1 1]);
        zpatch = ([0 0 0 0 0 1; 0 0 0 0 0 1; 1 1 1 1 0 1; 1 1 1 1 0 1]);
        XYZ = [xpatch(:) ypatch(:) zpatch(:)];
        XYZ = AxelRot(XYZ',-35.265,[1 0 0],[]);
        XYZ = AxelRot(XYZ,-45,[0 1 0],[]);
        xpatch = reshape(XYZ(1,:)',4,6);
        ypatch = reshape(XYZ(2,:)',4,6);
        zpatch = reshape(XYZ(3,:)',4,6);         

        for vv = 1:6
            if vv==1
                h = patch(xpatch(:,vv),ypatch(:,vv),zpatch(:,vv),'b','FaceAlpha',alp,'edgecolor','k');
            else
                h = patch(xpatch(:,vv),ypatch(:,vv),zpatch(:,vv),'w','edgecolor','k','FaceAlpha',0);                
            end
        end      
        axis off; view(30,22); daspect([1 1 1]); axis vis3d; camproj perspective; rotate3d on;
        
% graphic slice X
    ax_Bmid = axes('Units','pixels','Position',ax_Btop.Position+[spacings 0 0 0]);
        for vv = 1:6
            if vv==2
                h = patch(xpatch(:,vv),ypatch(:,vv),zpatch(:,vv),'b','FaceAlpha',alp,'edgecolor','k');
            else
                h = patch(xpatch(:,vv),ypatch(:,vv),zpatch(:,vv),'w','edgecolor','k','FaceAlpha',0);                
            end
        end         
        axis off; view(30,22); daspect([1 1 1]); axis vis3d; camproj perspective; rotate3d on;
        
% graphic slice Y
    ax_Bbot = axes('Units','pixels','Position',ax_Bmid.Position+[spacings 0 0 0]);
        for vv = 1:6
            if vv==6
                h = patch(xpatch(:,vv),ypatch(:,vv),zpatch(:,vv),'b','FaceAlpha',alp,'edgecolor','k');
            else
                h = patch(xpatch(:,vv),ypatch(:,vv),zpatch(:,vv),'w','edgecolor','k','FaceAlpha',0);                
            end
        end        
        axis off; view(30,22); daspect([1 1 1]); axis vis3d; camproj perspective; rotate3d on;
        
% graphic XYZ volume
    ax_Bmid = axes('Units','pixels','Position',ax_Bbot.Position+[160 0 0 0]);
        for vv = 1:6
            h = patch(xpatch(:,vv),ypatch(:,vv),zpatch(:,vv),'b');
            set(h,'edgecolor','k','FaceColor','b','FaceAlpha',alp/2)
        end   
        axis off; view(30,22); daspect([1 1 1]); axis vis3d; camproj perspective; rotate3d on   

%% ##################################### Distribution plots      
    xs = [50 170 290 430];
    ys = [550 380 NaN 210];
    xi = -1:0.01:1;
    bw = .1;
        
    idx = [1 1;2 1;3 1;4 1;1 2;2 2;3 2;4 2;1 4;2 4;3 4;4 4];
    mnames = {'Arena','Aligned','Tilted','Tilted'};
    max_lim = 0.03;
    for ii = 1:size(idx,1)
        maz = idx(ii,2);
        dim = idx(ii,1);
        if maz>2
            dnames = {'AB','AC','BC','ABC'};
        else
            dnames = {'XY','XZ','YZ','XYZ'};
        end
        fo = histDENSITY(obs_field_stab(:,dim,maz),xi,bw) ./ sum(~isnan(obs_field_stab(:,dim,maz)));
        fs = histDENSITY(shuff_field_stab(:,dim,maz),xi,bw) ./ sum(~isnan(shuff_field_stab(:,dim,maz)));

        if dim==4
            wid = 150;
        else
            wid = 110;
        end
        ax_A = axes('Units','pixels','Position',[xs(dim) ys(maz) wid 80]);   

        area(xi,fo,'FaceColor','b','FaceAlpha',.2); hold on;
        area(xi,fs,'FaceColor','r','FaceAlpha',.2);
        box off

        ax = gca;
        ax.YLim(2) = max_lim;
        
        if dim>1
            if dim==4
                ax.YLim(2) = .062;
            else
                ax.YTick = [];
                ax.YColor = 'none';
            end
        else
            ylabel('Proportion of cells');            
        end
        if maz<4
            ax.XTick = [];
        else
            xlabel('Correlation (r)')
        end
        
        p95 = prctile(shuff_field_stab(:,dim,maz),95);
        m50 = nanmedian(obs_field_stab(:,dim,maz));
        lin1 = line([p95 p95],ax.YLim,'Color','r');
        lin2 = line([m50 m50],ax.YLim,'Color','b');        

        [p,~,s] = ranksum(obs_field_stab(:,dim,maz),shuff_field_stab(:,dim,maz),'method','approximate','tail','right');
        if p<.001
            text(0.02,1.08,sprintf('z = %.1f, p < .001',s.zval),'Units','normalized','FontSize',6)
        else
            text(0.02,1.08,sprintf('z = %.1f, p = %.3f',s.zval,p),'Units','normalized','FontSize',6)        
        end
        text(0.02,0.97,sprintf('shuffle 95th = %.3f',p95),'Color','r','Units','normalized','FontSize',6,'HorizontalAlignment','left')   
        text(0.02,0.87,sprintf('median = %.3f',m50),'Color','b','Units','normalized','FontSize',6,'HorizontalAlignment','left')   

        set(gca,'LineWidth',flnw,'layer','top','FontSize',fsiz,'XColor','k'); % set the axes to be on top, set their thickness to flnw, set all fonts to size fsiz and make sure the axis and text are black
        set(gca,'yticklabel',num2str(get(gca,'ytick')','%.2f')) % change Ytick labels to have the same number of decimal places
        
        if dim==1 & maz==4
            a1 = patch([-1 -1 -1.1 -1.1],[-1 -1 -1.1 -1.1],[.7 .7 1]);
            a2 = patch([-1 -1 -1.1 -1.1],[-1 -1 -1.1 -1.1],[1 .7 .7]);            
            [~,o] = legendflex([a1,a2,lin1,lin2], {'Observed','Shuffle','Shuffle 95th','Observed median'}, 'ref', gca, 'xscale', 0.4, 'anchor', [3 8], 'buffer', [-100 -140], 'box', 'off', 'FontSize',6);
        end
        
    end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%                
%% ################################################################# %% Finish up and save
    paper_FIG_savefig(print_now,print_qual,gcf,sdir,dbstack);
    


































