%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% ################################################################# %% Created for
%
%     The place-cell representation of volumetric space in rats
%     Roddy M. Grieves, Selim Jedidi-Ayoub, Karyna Mishchanchuk, Anyi Liu, Sophie Renaudineau and Kate J. Jeffery
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% ################################################################# %% DESCRIPTION
% FIGS8_recording_stability small function to make Fig. S8
% Figure S8 shows the result of an analysis to determine if cells were stable between the first and second arena session
%
% USAGE:
%         Should be run via analysePCELLS
%
% INPUT:
%         No inputs, this is a script that relies on analysePCELLS to load data
%
% OUTPUT:
%    Will save a figure if print_now is set to 1 in analysePCELLS
%
% See also: analysePCELLS

% HISTORY:
% version 1.0.0, Release 30/09/19 Initial release
%
% Author: Roddy Grieves
% UCL, 26 Bedford Way
% eMail: r.grieves@ucl.ac.uk
% Copyright 2019 Roddy Grieves

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% ################################################################# %% INPUT ARGUMENTS CHECK
close all
disp(sprintf('Running paper_3_stability...'))
pnames = {'arena 1','aligned','tilted','tilted'};

if ~exist('stabmat','var')
    load('stabDATA.mat')
end
stab_sq1_idx = stabmat.part(:)==1;
stab_sq2_idx = stabmat.part(:)==3;
stab_la1_idx = stabmat.part(:)==2 & stabmat.lat_type==1;
stab_la2_idx = stabmat.part(:)==2 & stabmat.lat_type==2;

%% ##################################### %% Compare first and second square for a measure of stability
disp(sprintf('\tComparing first and last arena'))
    cell_indx = unique(stabmat.uci(stab_sq1_idx));
    obs_rec_stab = NaN(length(cell_indx),4);
    all_maps = cell(length(cell_indx),2);
    loopout = looper(length(cell_indx));
    for cc = 1:length(cell_indx) % for every cell
        cidx = stabmat.uci(:)==cell_indx(cc); % index for current cell

        for dd = 1:4 % for each dimension/projection
            dim_list = {'xy','xz','yz','xyz'};   
                
            % skip this cell if it is missing the part we want
            if ~any(cidx & stabmat.part==1) | ~any(cidx & stabmat.part==3)
                loopout = looper(loopout);
                continue
            end

            % get both projected half maps
            map1 = stabmat.(['map_' dim_list{dd}]){cidx & stabmat.part==1};
            map2 = stabmat.(['map_' dim_list{dd}]){cidx & stabmat.part==3}; 

            % if one of the maps are missing skip this cell
            if any([isempty(map1) isempty(map2)])
                loopout = looper(loopout);        
                continue
            end

            % if both maps have a firing rate less than 1Hz skip this cell
            if all([nanmax(map1(:))<1 nanmax(map2(:))<1])
                loopout = looper(loopout);        
                continue
            end

            all_maps{cc,1,dd} = map1;
            all_maps{cc,2,dd} = map2;         

            % compute correlation
            [r,p] = corr(map1(:),map2(:),'Type','Pearson','Rows','pairwise');
            obs_rec_stab(cc,dd) = r;
            loopout = looper(loopout);
        end
    end    

%% ##################################### %% Compare random first and second squares for a chance distribution
disp(sprintf('\tComparing first and last arena (shuffled)'))
    shuff = 1000;
    ncells = size(all_maps,1);
    shuff_rec_stab = NaN(shuff,4);
    loopout = looper(shuff);
    for cc = 1:shuff % for every shuffle
        for dd = 1:4 % for every projection dimension
            shuff_wait = true;
            while shuff_wait
                rindx = randperm(ncells,2); % get a random cell index for 2 cells
                map1 = all_maps{rindx(1),1,dd}; % get first half map for this projection {cell,half,dim}
                map2 = all_maps{rindx(2),2,dd}; % get second half map for this projection {cell,half,dim}
                if any([isempty(map1) isempty(map2)]) | all([nanmax(map1(:))<1 nanmax(map2(:))<1]) % if either map is missing or if both maps are <1Hz rerun while loop
                    continue
                end 
                [r,p] = corr(map1(:),map2(:),'Type','Pearson','Rows','pairwise'); % otherwise correlate the maps
                shuff_rec_stab(cc,dd) = r; % save the r value
                shuff_wait = false; % stop this while loop
            end
        end
        loopout = looper(loopout);
    end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% ################################################################# %% FIGURE
    fignow = figure('visible',fig_vis,'Position',[50,60,595,842]);
    set(gcf,'InvertHardCopy','off'); % this stops white lines being plotted black      
    set(gcf,'color','w'); % makes the background colour white
    colormap(jet(256)); % to make sure the colormap is not the horrible default one

    annotation('textbox','Units','pixels','Position',[1, 830, 300, 20],'string','a','FontSize',18,'LineStyle','none','interpreter','none'); 
    annotation('textbox','Units','pixels','Position',[1, 700, 300, 20],'string','b','FontSize',18,'LineStyle','none','interpreter','none');     
    
    annotation('textbox','Units','pixels','Position',[25, 675, 300, 20],'string',sprintf('Arena\nprojected'),'FontSize',fsiz+2,'LineStyle','none','interpreter','none'); 
    annotation('textbox','Units','pixels','Position',[360, 675, 300, 20],'string',sprintf('Arena\nvolume'),'FontSize',fsiz+2,'LineStyle','none','interpreter','none');     
    
%% ##################################### %% example 1st and 2nd arenas
%     dirs = {'Y:\Jeffery lab\Roddy G\Experiments\Lattice Maze\Data\Place cells\894_P9_32_HPC\171221\klustest3\kwiktint',12,1,894,171221;...
%             'Y:\Jeffery lab\Roddy G\Experiments\Lattice Maze\Data\Place cells\894_P9_32_HPC\171221\klustest3\kwiktint',12,1,894,171221;...
%             'Y:\Jeffery lab\Roddy G\Experiments\Lattice Maze\Data\Place cells\896_P11_32_HPC\180109\klustest3\kwiktint',11,4,896,180109;...
%             'Y:\Jeffery lab\Roddy G\Experiments\Lattice Maze\Data\Place cells\896_P11_32_HPC\180109\klustest3\kwiktint',11,4,896,180109};    
    dirs = {[sdir '\894_21122017.mat'],12,1,894,171221;...
            [sdir '\894_21122017.mat'],12,1,894,171221;...
            [sdir '\896_09012018.mat'],11,4,896,180109;...
            [sdir '\896_09012018.mat'],11,4,896,180109};    
        
    annotation('textbox','Units','pixels','Position',[100, 818, 300, 20],'string',sprintf('%s %s %s %s',num2str(dirs{1,4}),num2str(dirs{1,5}),num2str(dirs{1,2}),num2str(dirs{1,3})),'FontSize',fsiz,'LineStyle','none','interpreter','none'); 
    annotation('textbox','Units','pixels','Position',[350, 818, 300, 20],'string',sprintf('%s %s %s %s',num2str(dirs{3,4}),num2str(dirs{3,5}),num2str(dirs{3,2}),num2str(dirs{3,3})),'FontSize',fsiz,'LineStyle','none','interpreter','none'); 
                
    axpos = {[30 700 150 150],[130 700 150 150],[260 700 150 150],[360 700 150 150]};
    for dd = 1:size(dirs,1)   
        disp(sprintf('\t...cell: %d',dd));
        cd(sdir); 

        % collect the data we want
        if isodd(dd)
            load(dirs{dd,1},'-mat','sdata');  
            uci = ['r' sdata.rat_num '_' sdata.date '_t' num2str(dirs{dd,2}) '_c' num2str(dirs{dd,3})]; 
            ratemap = sdata.(uci).square1_3d.ratemap;
            lx2 = sdata.square1_3d.lxrm;
            ly2 = sdata.square1_3d.lyrm;
            lz2 = sdata.square1_3d.lzrm; 
        elseif iseven(dd)
            ratemap = sdata.(uci).square2_3d.ratemap;
            lx2 = sdata.square2_3d.lxrm;
            ly2 = sdata.square2_3d.lyrm;
            lz2 = sdata.square2_3d.lzrm;             
        end      

        % plot data
        ax_stab1 = axes('Units','pixels','Position',axpos{dd});
            ratemap(ratemap < 0.1*nanmax(ratemap(:))) = NaN;
            vol3d('cdata',ratemap,'texture','3D');
            alphamap('rampup');
            colormap(gca,'jet');
            caxis([0 nanmax(ratemap(:))]);

            hold on
            edg = [max(lx2)-min(lx2), max(ly2)-min(ly2), max(lz2)-min(lz2)];
            ori = [min(lx2), min(ly2), min(lz2)]; 
            plotcube(edg,ori,0,0,0.3);

            axis off
            pad_valr = 6;
            axis([min(lx2)-pad_valr,max(lx2)+pad_valr,min(ly2)-pad_valr,max(ly2)+pad_valr,min(lz2)-pad_valr,max(lz2)+pad_valr]);        
            view(30,45)
            daspect([1 1 1])
            camproj perspective
            rotate3d on 
    end    
    cd(sdir);

%% colorbar for 3D ratemap
    axc_cmap = axes('Units','pixels','Position',[axpos{end}(1)+160 axpos{end}(2)+40 10 80]);
        cmap = repmat(reshape(1:100,[],1),1,1,2);
        cmap(cmap < 0.2*nanmax(cmap(:))) = NaN;
        vol3d('cdata',cmap,'texture','3D');
        alphamap('rampup');
        colormap(gca,'jet');
        caxis([0.2*nanmax(cmap(:)) nanmax(cmap(:))]);
        axis xy on tight
        box on
        ax = gca;
        ax.XTick = [];
        ax.YTick = [20];
        ax.YTickLabel = {'10%'};
        text(0.5,-0.08,'0 Hz','FontSize',fsiz,'Units','normalized','HorizontalAlignment','center','Color','k');
        text(0.5,1.08,'Max','FontSize',fsiz,'Units','normalized','HorizontalAlignment','center','Color','k');    
    
%% ##################################### %% Correlation schematic
    xs = [100];
    spacings = 120;
    alp = .2;
    
%% Arena
% graphic slice Z
    ax_Btop = axes('Units','pixels','Position',[xs(1) 635 90 90]);
        xpatch = ([0 1 1 0 0 0; 1 1 0 0 1 1; 1 1 0 0 1 1; 0 1 1 0 0 0]);
        ypatch = ([0 0 1 1 0 0; 0 1 1 0 0 0; 0 1 1 0 1 1; 0 0 1 1 1 1]);
        zpatch = ([0 0 0 0 0 1; 0 0 0 0 0 1; 1 1 1 1 0 1; 1 1 1 1 0 1]).*0.2;
        for vv = 1:6
            h = patch(xpatch(:,vv),ypatch(:,vv),zpatch(:,vv),'w');
            set(h,'edgecolor','k','FaceColor','none')
        end      
        hold on
        surf([0 0; 1 1],[0 1; 0 1],[0 0; 0 0].*2,'FaceColor','b','FaceAlpha',alp); axis off; view(30,22); daspect([1 1 1]); axis vis3d; camproj perspective; rotate3d on    
 
% graphic XYZ volume
    ax_Bmid = axes('Units','pixels','Position',ax_Btop.Position+[320 0 0 0]);
        for vv = 1:6
            h = patch(xpatch(:,vv),ypatch(:,vv),zpatch(:,vv),'b');
            set(h,'edgecolor','k','FaceColor','b','FaceAlpha',alp/2)
        end   
        axis off; view(30,22); daspect([1 1 1]); axis vis3d; camproj perspective; rotate3d on   

%% ##################################### %% XY projection correlation distributions
    ax1 = axes('Units','pixels','Position',[50 500 250 130]);   
        xi = -1:0.01:1;
        bw = .1;
        fobs = histDENSITY(obs_rec_stab(:,1),xi,bw) ./ sum(~isnan(obs_rec_stab(:,1)));
        fshuff = histDENSITY(shuff_rec_stab(:,1),xi,bw) ./ sum(~isnan(shuff_rec_stab(:,1)));
        fobs3 = histDENSITY(obs_rec_stab(:,4),xi,bw) ./ sum(~isnan(obs_rec_stab(:,4)));
        fshuff3 = histDENSITY(shuff_rec_stab(:,4),xi,bw) ./ sum(~isnan(shuff_rec_stab(:,4)));

        area(xi,fshuff,'FaceColor','r','FaceAlpha',.2); hold on;
        area(xi,fobs,'FaceColor','b','FaceAlpha',.2);
        xlabel('Correlation (r)')
        ylabel('Proportion of cells') 
        box off

        ax = gca;
        max_lim = 0.035;
        ax.YLim(2) = max_lim;
        p95 = prctile(shuff_rec_stab(:,1),95);
        m50 = nanmedian(obs_rec_stab(:,1));
        line([p95 p95],ax.YLim,'Color','r');
        line([m50 m50],ax.YLim,'Color','b');

        [p,~,s] = ranksum(obs_rec_stab(:,1),shuff_rec_stab(:,1),'method','approximate','tail','right');
        if p<.001
            text(0.02,1.08,sprintf('z = %.1f, p < .001',s.zval),'Units','normalized','FontSize',6)
        else
            text(0.02,1.08,sprintf('z = %.1f, p = %.3f',s.zval,p),'Units','normalized','FontSize',6)        
        end
        text(0.02,0.97,sprintf('shuffle 95th = %.3f',p95),'Color','r','Units','normalized','FontSize',6,'HorizontalAlignment','left')   
        text(0.02,0.87,sprintf('median = %.3f',m50),'Color','b','Units','normalized','FontSize',6,'HorizontalAlignment','left')

%% ##################################### %% XYZ correlation distributions
    ax2 = axes('Units','pixels','Position',ax1.Position+[280 0 0 0]);   
        area(xi,fshuff3,'FaceColor','r','FaceAlpha',.2); hold on;
        area(xi,fobs3,'FaceColor','b','FaceAlpha',.2);
        xlabel('3D Correlation (r)')

        ax = gca;
        ax.YLim(2) = max_lim;
        ax.YTick = [];
        ax.YColor = 'none';
        box off
        p95 = prctile(shuff_rec_stab(:,4),95);
        m50 = nanmedian(obs_rec_stab(:,4));
        line([p95 p95],ax.YLim,'Color','r');
        line([m50 m50],ax.YLim,'Color','b');

        [p,~,s] = ranksum(obs_rec_stab(:,4),shuff_rec_stab(:,4),'method','approximate','tail','right');
        if p<.001
            text(0.02,1.08,sprintf('z = %.1f, p < .001',s.zval),'Units','normalized','FontSize',6)
        else
            text(0.02,1.08,sprintf('z = %.1f, p = %.3f',s.zval,p),'Units','normalized','FontSize',6)        
        end
        text(0.02,0.97,sprintf('shuffle 95th = %.3f',p95),'Color','r','Units','normalized','FontSize',6,'HorizontalAlignment','left')   
        text(0.02,0.87,sprintf('median = %.3f',m50),'Color','b','Units','normalized','FontSize',6,'HorizontalAlignment','left')

%% ##################################### %% Correlation distributions for XZ and YZ projections        
    xs = [50];
    spacings = 160;
    alp = .2;
    
%% Arena
% graphic slice X
    ax_Bmid = axes('Units','pixels','Position',[xs(1) 365 90 90]);
        for vv = 1:6
            h = patch(xpatch(:,vv),ypatch(:,vv),zpatch(:,vv),'w');
            set(h,'edgecolor','k','FaceColor','none')
        end         
        hold on;
        surf([0 0;0 0],[0 0; 1 1],[0 1; 0 1].*.2,'FaceColor','b','FaceAlpha',alp); axis off; view(30,22); daspect([1 1 1]); axis vis3d; camproj perspective; rotate3d on    
        
% graphic slice Y
    ax_Bbot = axes('Units','pixels','Position',ax_Bmid.Position+[spacings 0 0 0]);
        for vv = 1:6
            h = patch(xpatch(:,vv),ypatch(:,vv),zpatch(:,vv),'w');
            set(h,'edgecolor','k','FaceColor','none','FaceAlpha',.5)
        end        
        hold on
        surf([1 0; 1 0],[1 1;1 1],[0 0; 1 1].*.2,'FaceColor','b','FaceAlpha',alp); axis off; view(30,22); daspect([1 1 1]); axis vis3d; camproj perspective; rotate3d on   

% %% ##################################### Correlation distributions for projections          
    xs = [50 190];
    ys = [250 250];
    
    idx = [2 3];
    max_lim = 0.03;
    for ii = 1:length(idx)
        col = idx(ii);
        fo = histDENSITY(obs_rec_stab(:,col),xi,bw) ./ sum(~isnan(obs_rec_stab(:,col)));
        fs = histDENSITY(shuff_rec_stab(:,col),xi,bw) ./ sum(~isnan(shuff_rec_stab(:,col)));

        ax_A = axes('Units','pixels','Position',[xs(ii) ys(ii) 120 110]);   
            area(xi,fo,'FaceColor','b','FaceAlpha',.2); hold on;
            area(xi,fs,'FaceColor','r','FaceAlpha',.2);
            box off
            ax = gca;
            ax.YLim(2) = max_lim;
        
            if ii==2
                ax.YTick = [];
                ax.YColor = 'none';
            else
                ylabel('Proportion of cells');            
            end
            xlabel('Correlation (r)')
        
            p95 = prctile(shuff_rec_stab(:,col),95);
            m50 = nanmedian(obs_rec_stab(:,col));
            lin1 = line([p95 p95],ax.YLim,'Color','r');
            lin2 = line([m50 m50],ax.YLim,'Color','b');        

            [p,~,s] = ranksum(obs_rec_stab(:,col),shuff_rec_stab(:,col),'method','approximate','tail','right');
            if p<.001
                text(0.02,1.08,sprintf('z = %.1f, p < .001',s.zval),'Units','normalized','FontSize',6)
            else
                text(0.02,1.08,sprintf('z = %.1f, p = %.3f',s.zval,p),'Units','normalized','FontSize',6)        
            end
            text(0.02,0.97,sprintf('shuffle 95th = %.3f',p95),'Color','r','Units','normalized','FontSize',6,'HorizontalAlignment','left')   
            text(0.02,0.87,sprintf('median = %.3f',m50),'Color','b','Units','normalized','FontSize',6,'HorizontalAlignment','left')   

            set(gca,'LineWidth',flnw,'layer','top','FontSize',fsiz,'XColor','k'); % set the axes to be on top, set their thickness to flnw, set all fonts to size fsiz and make sure the axis and text are black
            set(gca,'yticklabel',num2str(get(gca,'ytick')','%.2f')) % change Ytick labels to have the same number of decimal places
    end
    a1 = patch([-1 -1 -1.1 -1.1],[-1 -1 -1.1 -1.1],[.7 .7 1]);
    a2 = patch([-1 -1 -1.1 -1.1],[-1 -1 -1.1 -1.1],[1 .7 .7]);            
    [~,o] = legendflex([a1,a2,lin1,lin2], {'Observed','Shuffle','Shuffle 95th','Observed median'}, 'ref', gca, 'xscale', 0.4, 'anchor', [3 8], 'buffer', [10 -20], 'box', 'off', 'FontSize',6);    

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%                
%% ################################################################# %% Finish up and save
    paper_FIG_savefig(print_now,print_qual,gcf,sdir,dbstack);
    



















    
