%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% ################################################################# %% Created for
%
%     The place-cell representation of volumetric space in rats
%     Roddy M. Grieves, Selim Jedidi-Ayoub, Karyna Mishchanchuk, Anyi Liu, Sophie Renaudineau and Kate J. Jeffery
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% ################################################################# %% DESCRIPTION
% FIGS10_zingg_results small function to make Fig. S10
% Figure S10 shows the result of downsampling trajectories
%
% USAGE:
%         Should be run via analysePCELLS
%
% INPUT:
%         No inputs, this is a script that relies on analysePCELLS to load data
%
% OUTPUT:
%    Will save a figure if print_now is set to 1 in analysePCELLS
%
% See also: analysePCELLS

% HISTORY:
% version 1.0.0, Release 30/09/19 Initial release
%
% Author: Roddy Grieves
% UCL, 26 Bedford Way
% eMail: r.grieves@ucl.ac.uk
% Copyright 2019 Roddy Grieves

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% ################################################################# %% INPUT ARGUMENTS CHECK & FIGURE
    fig2now = figure('visible',fig_vis,'Position',[50,60,595,842]);
    set(gcf,'InvertHardCopy','off'); % this stops white lines being plotted black      
    set(gcf,'color','w'); % makes the background colour white
    colormap(jet(256)); % to make sure the colormap is not the horrible default one

    annotation('textbox','Units','pixels','Position',[5, 820, 300, 20],'string','a','FontSize',18,'LineStyle','none','interpreter','none'); 
    annotation('textbox','Units','pixels','Position',[5, 390, 300, 20],'string','b','FontSize',18,'LineStyle','none','interpreter','none'); 
    annotation('textbox','Units','pixels','Position',[5, 200, 300, 20],'string','c','FontSize',18,'LineStyle','none','interpreter','none'); 

%% ##################################### % Example of horizontal/vertical downsample
    dirs = {[],[sdir '\850_13032017.mat'],5,4,850,130317};
    cd(sdir);
    load(dirs{2},'-mat','sdata');      

    % get the spike data
    part_now3 = 'lattice_3d';
    pox = sdata.(part_now3).pox;
    poy = sdata.(part_now3).poy;
    poz = sdata.(part_now3).poz;     
    lx = sdata.(part_now3).lx;
    ly = sdata.(part_now3).ly;
    lz = sdata.(part_now3).lz; 

    %% Determine time spent heading in every heading    
    % calculate change in position along each axis
    pxd = padarray(pox,25,NaN,'both');
    pxd = [circshift(pxd(:),25) circshift(pxd(:),-25)];
    pxd = pxd(26:end-25,1)-pxd(26:end-25,2);
    pxd = pxd./50;
    pyd = padarray(poy,25,NaN,'both');
    pyd = [circshift(pyd(:),25) circshift(pyd(:),-25)];
    pyd = pyd(26:end-25,1)-pyd(26:end-25,2);
    pyd = pyd./50;
    pzd = padarray(poz,25,NaN,'both');
    pzd = [circshift(pzd(:),25) circshift(pzd(:),-25)];
    pzd = pzd(26:end-25,1)-pzd(26:end-25,2);
    pzd = pzd./50;

    % normalise these vectors
    V = [pxd(:) pyd(:) pzd(:)];
    V = normr(V);

    % azimuth and pitch
    [azi,pit,~] = cart2sph(V(:,1),V(:,2),V(:,3));        

    % seperate data into vertical and horizontal movements
    pcut = deg2rad(30); % 30 degrees seperates a sphere into two equal parts
    utime = pit>pcut | pit<-pcut; % points where the animal is heading up or down
    hvtime = utime;
    nindx = find(~utime); % all the points where the animal is moving vertically
    nindx = nindx(randperm(numel(nindx),sum(utime))); % collect same number of points where the animal is moving horizontally
    hvtime(nindx) = 1; % combine them in hvtime

    % create new data with half vertical and half horizontal points
    poxhv = pox;
    poyhv = poy;
    pozhv = poz;
    poxhv(~hvtime) = NaN;
    poyhv(~hvtime) = NaN;
    pozhv(~hvtime) = NaN;

%% three plots, horizontal only, vertical only, 50% mix
    ax_P1 = axes('Units','pixels','Position',[50 680 120 120]); 
        poxn = pox; poxn(~utime) = NaN;
        poyn = poy; poyn(~utime) = NaN;
        pozn = poz; pozn(~utime) = NaN;
        cc = cline(poxn,poyn,pozn,pozn); hold on
        set(cc,'LineWidth',1);
        line(lx,ly,lz,'Color',[.5 .5 .5 .5]);
        axis([min(pox),max(pox),min(poy),max(poy),min(poz),max(poz)])
        axis equal off
        vnow = [-38,22];
        view(vnow)
        title(sprintf('Vertical movements (%.1f s)',sum(utime)*(1/50)),'FontWeight','normal','FontSize',fsiz)
        caxis([min(pox) max(pox)])
    
    ax_P2 = axes('Units','pixels','Position',ax_P1.Position+[180 0 0 0]); 
        poxn = pox; poxn(utime) = NaN;
        poyn = poy; poyn(utime) = NaN;
        pozn = poz; pozn(utime) = NaN;
        cc = cline(poxn,poyn,pozn,pozn); hold on
        set(cc,'LineWidth',1);        
        line(lx,ly,lz,'Color',[.5 .5 .5 .5]);        
        axis([min(pox),max(pox),min(poy),max(poy),min(poz),max(poz)])
        axis equal off
        view(vnow)
        title(sprintf('Horizontal movements (%.1f s)',sum(~utime)*(1/50)),'FontWeight','normal','FontSize',fsiz)
        caxis([min(pox) max(pox)])
    
    ax_P3 = axes('Units','pixels','Position',ax_P2.Position+[180 0 0 0]); 
        cc = cline(poxhv,poyhv,pozhv,pozhv); hold on
        set(cc,'LineWidth',1);        
        line(lx,ly,lz,'Color',[.5 .5 .5 .5]);        
        axis([min(pox),max(pox),min(poy),max(poy),min(poz),max(poz)])
        axis equal off
        view(vnow)
        title(sprintf('H/V downsample (%.1f s)',sum(hvtime)*(1/50)),'FontWeight','normal','FontSize',fsiz)
        caxis([min(pox) max(pox)])
    
    axc = axes('Units','pixels','Position',[ax_P3.Position(1)+150 ax_P3.Position(2) 10 ax_P3.Position(4)]);
        imagesc(((1:256)'));
        colormap(axc,'jet')
        axis xy off
        text(0.5,-0.08,'0','FontSize',fsiz,'Units','normalized','HorizontalAlignment','center','Color','k');
        text(0.5,1.08,'Max','FontSize',fsiz,'Units','normalized','HorizontalAlignment','center','Color','k');

%% ##################################### Example of in/out downsample
    %% filter data to match inside and outside coverage
    slength = nanmax(lx)-nanmin(lx);
    length2 = nthroot(slength^3/2,3);
    cent = [nanmean(lx) nanmean(ly) nanmean(lz)];
    inner_indx = pox>cent(1)-(length2/2) & pox<cent(1)+(length2/2) & poy>cent(2)-(length2/2) & poy<cent(2)+(length2/2) & poz>cent(3)-(length2/2) & poz<cent(3)+(length2/2);        
        
    time_in = sum(inner_indx);
    time_out = sum(~inner_indx);
    inner_index = find(inner_indx);
    outer_index = find(~inner_indx); 
    if time_in > time_out % if the animal spent more time inside than outside
        iotime = ~inner_indx;
        iotime(inner_index(randperm(numel(inner_index),numel(outer_index)))) = true;       
    elseif time_in <= time_out % if the animal spent more time outside than inside
        iotime = inner_indx;
        iotime(outer_index(randperm(numel(outer_index),numel(inner_index)))) = true;
    end   
    
    % create new data with half inner and half outer points
    poxio = pox;
    poyio = poy;
    pozio = poz;
    poxio(~iotime) = NaN;
    poyio(~iotime) = NaN;
    pozio(~iotime) = NaN;        
        
%% three plots, inside only, outside only, 50% mix
    ax_IO1 = axes('Units','pixels','Position',[50 540 120 120]); 
        poxn = pox; poxn(~inner_indx) = NaN;
        poyn = poy; poyn(~inner_indx) = NaN;
        pozn = poz; pozn(~inner_indx) = NaN;
        cc = cline(poxn,poyn,pozn,pozn); hold on
        set(cc,'LineWidth',1);
        line(lx,ly,lz,'Color',[.5 .5 .5 .5]);
        axis([min(pox),max(pox),min(poy),max(poy),min(poz),max(poz)])
        axis equal off
        vnow = [-38,22];
        view(vnow)
        title(sprintf('Inside movements (%.1f s)',sum(inner_indx)*(1/50)),'FontWeight','normal','FontSize',fsiz)
        caxis([min(pox) max(pox)])
    
    ax_IO2 = axes('Units','pixels','Position',ax_IO1.Position+[180 0 0 0]); 
        poxn = pox; poxn(inner_indx) = NaN;
        poyn = poy; poyn(inner_indx) = NaN;
        pozn = poz; pozn(inner_indx) = NaN;
        cc = cline(poxn,poyn,pozn,pozn); hold on
        set(cc,'LineWidth',1);        
        line(lx,ly,lz,'Color',[.5 .5 .5 .5]);        
        axis([min(pox),max(pox),min(poy),max(poy),min(poz),max(poz)])
        axis equal off
        view(vnow)
        title(sprintf('Outside movements (%.1f s)',sum(~inner_indx)*(1/50)),'FontWeight','normal','FontSize',fsiz)
        caxis([min(pox) max(pox)])
    
    ax_IO3 = axes('Units','pixels','Position',ax_IO2.Position+[180 0 0 0]); 
        cc = cline(poxio,poyio,pozio,pozio); hold on
        set(cc,'LineWidth',1);        
        line(lx,ly,lz,'Color',[.5 .5 .5 .5]);        
        axis([min(pox),max(pox),min(poy),max(poy),min(poz),max(poz)])
        axis equal off
        view(vnow)
        title(sprintf('I/O downsample (%.1f s)',sum(iotime)*(1/50)),'FontWeight','normal','FontSize',fsiz)        
        caxis([min(pox) max(pox)])
        
%% ##################################### Example of up/down downsample
    mid_z = mean([nanmax(lz) nanmin(lz)]);
    top_indx = poz>mid_z;            

    %% filter data to match top and bottom coverage
    time_up = sum(top_indx);
    time_down = sum(~top_indx);
    upper_index = find(top_indx);
    lower_index = find(~top_indx); 
    if time_up > time_down % if the animal spent more time up than down
        udtime = ~top_indx;
        udtime(upper_index(randperm(numel(upper_index),numel(lower_index)))) = true;       
    elseif time_up <= time_down % if the animal spent more time down than up
        udtime = top_indx;
        udtime(lower_index(randperm(numel(lower_index),numel(upper_index)))) = true;
    end
        
    % create new data with half vertical and half horizontal points
    poxud = pox;
    poyud = poy;
    pozud = poz;
    poxud(~udtime) = NaN;
    poyud(~udtime) = NaN;
    pozud(~udtime) = NaN;

%% three plots, inside only, outside only, 50% mix
    ax_UD1 = axes('Units','pixels','Position',[50 400 120 120]); 
        poxn = pox; poxn(~top_indx) = NaN;
        poyn = poy; poyn(~top_indx) = NaN;
        pozn = poz; pozn(~top_indx) = NaN;
        cc = cline(poxn,poyn,pozn,pozn); hold on
        set(cc,'LineWidth',1);
        line(lx,ly,lz,'Color',[.5 .5 .5 .5]);
        axis([min(pox),max(pox),min(poy),max(poy),min(poz),max(poz)])
        axis equal off
        vnow = [-38,22];
        view(vnow)
        title(sprintf('Top movements (%.1f s)',sum(top_indx)*(1/50)),'FontWeight','normal','FontSize',fsiz)
        caxis([min(pox) max(pox)])
    
    ax_UD2 = axes('Units','pixels','Position',ax_UD1.Position+[180 0 0 0]); 
        poxn = pox; poxn(top_indx) = NaN;
        poyn = poy; poyn(top_indx) = NaN;
        pozn = poz; pozn(top_indx) = NaN;
        cc = cline(poxn,poyn,pozn,pozn); hold on
        set(cc,'LineWidth',1);        
        line(lx,ly,lz,'Color',[.5 .5 .5 .5]);        
        axis([min(pox),max(pox),min(poy),max(poy),min(poz),max(poz)])
        axis equal off
        view(vnow)
        title(sprintf('Bottom movements (%.1f s)',sum(~top_indx)*(1/50)),'FontWeight','normal','FontSize',fsiz)
        caxis([min(pox) max(pox)])
    
    ax_UD3 = axes('Units','pixels','Position',ax_UD2.Position+[180 0 0 0]); 
        cc = cline(poxud,poyud,pozud,pozud); hold on
        set(cc,'LineWidth',1);        
        line(lx,ly,lz,'Color',[.5 .5 .5 .5]);        
        axis([min(pox),max(pox),min(poy),max(poy),min(poz),max(poz)])
        axis equal off
        view(vnow)
        title(sprintf('T/B downsample (%.1f s)',sum(udtime)*(1/50)),'FontWeight','normal','FontSize',fsiz)        
        caxis([min(pox) max(pox)])
   
%% ##################################### spatial information aligned XYZ
    ax_C1 = axes('Units','pixels','Position',[50 250 120 110]); 
        sinfo = tdata.dsamps_spatinfo_obs(pc_idx_la1,:);
        sinfo(~any(sinfo,2),:) = [];
        sinfo = sinfo ./ nansum(sinfo,2);

        [ds,gs] = vectorDATAGROUP([],sinfo(:,3),sinfo(:,2),sinfo(:,1));
        boxplot2(ds,gs,'linewidth',flnw,'boxcolor',{colx,coly,colz},'boxalpha',0,'dots',1,'dotsize',10,'dotcolor',{colx,coly,colz},'dotalpha',0.8);

        box off
        ax = gca;
        ax.XTick = [1:3];
        ax.XTickLabel = {'X' 'Y' 'Z'};  
        ax.XLim = [0.5 3.5];
        ax.YLim = [0 0.8];
        ynow = ax.YLim;
        yy = ylabel('Proportion of information');
        xx = xlabel('Maze axis');        
        set(gca,'LineWidth',flnw,'layer','top','FontSize',fsiz,'XColor','k','YColor','k'); % set the axes to be on top, set their thickness to flnw, set all fonts to size fsiz and make sure the axis and text are black        
        set(gca,'yticklabel',num2str(get(gca,'ytick')','%.1f')) % change Ytick labels to have the same number of decimal places   
        title('Observed','FontWeight','normal','FontSize',fsiz)
        text(0.05,0.1,sprintf('N = %d',size(sinfo,1)),'FontSize',fsiz,'Units','normalized')

            disp('Aligned spatial info XYZ:')        
            stats_and_bars(ds,gs,[],flnw,fsiz)
            ax.YLim = ynow;
        
    ax_HV = axes('Units','pixels','Position',ax_C1.Position+[130 0 0 0]); 
        sinfo = tdata.dsamps_spatinfo(pc_idx_la1,:);
        sinfo(~any(sinfo,2),:) = [];
        sinfo = sinfo ./ nansum(sinfo,2);

        [ds,gs] = vectorDATAGROUP([],sinfo(:,3),sinfo(:,2),sinfo(:,1));
        boxplot2(ds,gs,'linewidth',flnw,'boxcolor',{colx,coly,colz},'boxalpha',0,'dots',1,'dotsize',10,'dotcolor',{colx,coly,colz},'dotalpha',0.8);

        box off
        ax = gca;
        ax.XTick = [1:3];
        ax.XTickLabel = {'X' 'Y' 'Z'};  
        ax.YTickLabel = {};
        xx = xlabel('Maze axis');
        ax.XLim = [0.5 3.5];
        ax.YLim = ynow;
        set(gca,'LineWidth',flnw,'layer','top','FontSize',fsiz,'XColor','k','YColor','none'); % set the axes to be on top, set their thickness to flnw, set all fonts to size fsiz and make sure the axis and text are black        
        title('H/V downsample','FontWeight','normal','FontSize',fsiz)        
        disp('Aligned spatial info XYZ downsample:')                
        stats_and_bars(ds,gs,[],flnw,fsiz)    
        ax.YLim = ynow;

    ax_IO = axes('Units','pixels','Position',ax_HV.Position+[130 0 0 0]); 
        sinfo = tdata.dsamps_spatinfo_io(pc_idx_la1,:);
        sinfo(~any(sinfo,2),:) = [];
        sinfo = sinfo ./ nansum(sinfo,2);

        [ds,gs] = vectorDATAGROUP([],sinfo(:,3),sinfo(:,2),sinfo(:,1));
        boxplot2(ds,gs,'linewidth',flnw,'boxcolor',{colx,coly,colz},'boxalpha',0,'dots',1,'dotsize',10,'dotcolor',{colx,coly,colz},'dotalpha',0.8);

        box off
        ax = gca;
        ax.XTick = [1:3];
        ax.XTickLabel = {'X' 'Y' 'Z'};  
        ax.YTickLabel = {};
        xx = xlabel('Maze axis');
        ax.XLim = [0.5 3.5];
        ax.YLim = ynow;
        set(gca,'LineWidth',flnw,'layer','top','FontSize',fsiz,'XColor','k','YColor','none'); % set the axes to be on top, set their thickness to flnw, set all fonts to size fsiz and make sure the axis and text are black        
        title('I/O downsample','FontWeight','normal','FontSize',fsiz)        
        disp('Aligned spatial info XYZ downsample:')                
        stats_and_bars(ds,gs,[],flnw,fsiz)    
        ax.YLim = ynow;        
        
    ax_TB = axes('Units','pixels','Position',ax_IO.Position+[130 0 0 0]); 
        sinfo = tdata.dsamps_spatinfo_ud(pc_idx_la1,:);
        sinfo(~any(sinfo,2),:) = [];
        sinfo = sinfo ./ nansum(sinfo,2);

        [ds,gs] = vectorDATAGROUP([],sinfo(:,3),sinfo(:,2),sinfo(:,1));
        boxplot2(ds,gs,'linewidth',flnw,'boxcolor',{colx,coly,colz},'boxalpha',0,'dots',1,'dotsize',10,'dotcolor',{colx,coly,colz},'dotalpha',0.8);

        box off
        ax = gca;
        ax.XTick = [1:3];
        ax.XTickLabel = {'X' 'Y' 'Z'};  
        ax.YTickLabel = {};
        xx = xlabel('Maze axis');
        ax.XLim = [0.5 3.5];
        ax.YLim = ynow;
        set(gca,'LineWidth',flnw,'layer','top','FontSize',fsiz,'XColor','k','YColor','none'); % set the axes to be on top, set their thickness to flnw, set all fonts to size fsiz and make sure the axis and text are black        
        title('T/B downsample','FontWeight','normal','FontSize',fsiz)        
        disp('Aligned spatial info XYZ downsample:')                
        stats_and_bars(ds,gs,[],flnw,fsiz)    
        ax.YLim = ynow;            
           
%% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% %% slice correlation with distance XYZ
    ax_C1 = axes('Units','pixels','Position',[50 70 120 110]);         
        cinfo = cell2mat(tdata.dsamps_autovecs_obs(pc_idx_la1));
        cinfox = cinfo(1:3:end,:);
        cinfoy = cinfo(2:3:end,:);
        cinfoz = cinfo(3:3:end,:);
        
        xm = nanmean(cinfox,1);
        xs = nanstd(cinfox,[],1) ./ sqrt(sum(~isnan(cinfox),1));
        ym = nanmean(cinfoy,1);
        ys = nanstd(cinfoy,[],1) ./ sqrt(sum(~isnan(cinfoy),1));        
        zm = nanmean(cinfoz,1);
        zs = nanstd(cinfoz,[],1) ./ sqrt(sum(~isnan(cinfoz),1));        
        xi = (-30:30).*5;
        
        e1 = errorbar(xi,xm,xs,'Color',colx); hold on;
        e2 = errorbar(xi,ym,ys,'Color',coly);         
        e3 = errorbar(xi,zm,zs,'Color',colz);         
        
        ax = gca;
        ax.XLim = [0 100];
        ax.YLim = [0 0.8];
        ynow = ax.YLim;
        yy = ylabel('Correlation');   
        xlabel('Distance (cm)')      
        box off
        title('Observed','FontWeight','normal','FontSize',fsiz)        
        ax.YGrid = 'on';        
        set(gca,'LineWidth',flnw,'layer','top','FontSize',fsiz,'XColor','k','YColor','k'); % set the axes to be on top, set their thickness to flnw, set all fonts to size fsiz and make sure the axis and text are black        
        set(gca,'yticklabel',num2str(get(gca,'ytick')','%.1f')) % change Ytick labels to have the same number of decimal places                

    ax_hv = axes('Units','pixels','Position',ax_C1.Position+[130 0 0 0]);         
        cinfo = cell2mat(tdata.dsamps_autovecs(pc_idx_la1));
        cinfox = cinfo(1:3:end,:);
        cinfoy = cinfo(2:3:end,:);
        cinfoz = cinfo(3:3:end,:);
        
        xm = nanmean(cinfox,1);
        xs = nanstd(cinfox,[],1) ./ sqrt(sum(~isnan(cinfox),1));
        ym = nanmean(cinfoy,1);
        ys = nanstd(cinfoy,[],1) ./ sqrt(sum(~isnan(cinfoy),1));        
        zm = nanmean(cinfoz,1);
        zs = nanstd(cinfoz,[],1) ./ sqrt(sum(~isnan(cinfoz),1));        
        xi = (-30:30).*5;
        
        e1 = errorbar(xi,xm,xs,'Color',colx); hold on;
        e2 = errorbar(xi,ym,ys,'Color',coly);         
        e3 = errorbar(xi,zm,zs,'Color',colz);         
        
        ax = gca;
        ax.XLim = [0 100];
        ax.YLim = [0 0.8];
        ynow = ax.YLim;
        yy = ylabel('Correlation');   
        xlabel('Distance (cm)')      
        box off
        title('H/V downsample','FontWeight','normal','FontSize',fsiz)        
        ax.YGrid = 'on';        
        set(gca,'LineWidth',flnw,'layer','top','FontSize',fsiz,'XColor','k','YColor','none'); % set the axes to be on top, set their thickness to flnw, set all fonts to size fsiz and make sure the axis and text are black        
        set(gca,'yticklabel',num2str(get(gca,'ytick')','%.1f')) % change Ytick labels to have the same number of decimal places                
        
    ax_io = axes('Units','pixels','Position',ax_hv.Position+[130 0 0 0]);         
        cinfo = cell2mat(tdata.dsamps_autovecs_io(pc_idx_la1));
        cinfox = cinfo(1:3:end,:);
        cinfoy = cinfo(2:3:end,:);
        cinfoz = cinfo(3:3:end,:);
        
        xm = nanmean(cinfox,1);
        xs = nanstd(cinfox,[],1) ./ sqrt(sum(~isnan(cinfox),1));
        ym = nanmean(cinfoy,1);
        ys = nanstd(cinfoy,[],1) ./ sqrt(sum(~isnan(cinfoy),1));        
        zm = nanmean(cinfoz,1);
        zs = nanstd(cinfoz,[],1) ./ sqrt(sum(~isnan(cinfoz),1));        
        xi = (-30:30).*5;
        
        e1 = errorbar(xi,xm,xs,'Color',colx); hold on;
        e2 = errorbar(xi,ym,ys,'Color',coly);         
        e3 = errorbar(xi,zm,zs,'Color',colz);         
        
        ax = gca;
        ax.XLim = [0 100];
        ax.YLim = [0 0.8];
        ynow = ax.YLim;
        yy = ylabel('Correlation');   
        xlabel('Distance (cm)')      
        box off
        title('I/O downsample','FontWeight','normal','FontSize',fsiz)        
        ax.YGrid = 'on';        
        set(gca,'LineWidth',flnw,'layer','top','FontSize',fsiz,'XColor','k','YColor','none'); % set the axes to be on top, set their thickness to flnw, set all fonts to size fsiz and make sure the axis and text are black        
        set(gca,'yticklabel',num2str(get(gca,'ytick')','%.1f')) % change Ytick labels to have the same number of decimal places                
        
    ax_ud = axes('Units','pixels','Position',ax_io.Position+[130 0 0 0]);         
        cinfo = cell2mat(tdata.dsamps_autovecs_ud(pc_idx_la1));
        cinfox = cinfo(1:3:end,:);
        cinfoy = cinfo(2:3:end,:);
        cinfoz = cinfo(3:3:end,:);
        
        xm = nanmean(cinfox,1);
        xs = nanstd(cinfox,[],1) ./ sqrt(sum(~isnan(cinfox),1));
        ym = nanmean(cinfoy,1);
        ys = nanstd(cinfoy,[],1) ./ sqrt(sum(~isnan(cinfoy),1));        
        zm = nanmean(cinfoz,1);
        zs = nanstd(cinfoz,[],1) ./ sqrt(sum(~isnan(cinfoz),1));        
        xi = (-30:30).*5;
        
        e1 = errorbar(xi,xm,xs,'Color',colx); hold on;
        e2 = errorbar(xi,ym,ys,'Color',coly);         
        e3 = errorbar(xi,zm,zs,'Color',colz);         
        
        ax = gca;
        ax.XLim = [0 100];
        ax.YLim = [0 0.8];
        ynow = ax.YLim;
        yy = ylabel('Correlation');   
        xlabel('Distance (cm)')      
        box off
        title('T/B downsample','FontWeight','normal','FontSize',fsiz)        
        ax.YGrid = 'on';        
        set(gca,'LineWidth',flnw,'layer','top','FontSize',fsiz,'XColor','k','YColor','none'); % set the axes to be on top, set their thickness to flnw, set all fonts to size fsiz and make sure the axis and text are black        
        set(gca,'yticklabel',num2str(get(gca,'ytick')','%.1f')) % change Ytick labels to have the same number of decimal places                
        
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%                
%% ################################################################# %% Finish up and save
    paper_FIG_savefig(print_now,print_qual,gcf,sdir,dbstack);
    
            





































