function Arrow = farrow(x0, y0, z0, x1, y1, z1, color, width)
    % Purpose: Light-weight arrow function for 3d animations.
    
    % Output: Line-object 'approximation' of a 3d arrow.
    
    % Input (required): head coordinates (x0, y0, z0) and 
    % . . . . . . . . . tail coordinates (x0, y0, y0)
       
    % Input (optional): arrow color and arrow width (color, width)
    
    % Good: --- Fast!
  
    % Bad:
    % --- Arrow-head may be incomplete (if arrow is big)
    % --- Arrow-head may get distorted (if plot dimensions are uneven)
    
    % Options:
    % --- Line 18: Increase 'detail' to 500 for better arrow
    % --- Line 19: Change arrow default color
    % --- Line 20: Change arrow default width
    
    % How function works
    % --- Start Buffers (X, Y, Z)
    % --- Set first entry of buffers as line between tail and head
    
    % --- Loop (Generate buffer-data for 50 random lines along arrow-head):
    % --- --- Generate directional vector (u) from tail to head.
    % --- --- Generate random vector (random_vec)
    % --- --- Cross product [directional vector] x [random vector]
    % --- --- --- to get radial vector (rad)
    % --- --- Normalize radial vector
    % --- --- Add data for line along cone to buffer (X, Y, Z)
    % --- Generate line object with [X, Y, Z] data
    % --- Return 'Arrow'
    
    % How to improve:
    % --- Better math could be used to generate the cone more efficiently
    % --- and more reliably (something better than random cross-product)
    % --- --- This would fix problems with reliability.
       
    detail = 500;                                % Number of cone lines
    arrowcolor = [.8 .2 .2];                    % Arrow color
    arrowwidth = 1.5;                             % Arrow width
    
    if nargin > 6                               % If no 7th argument:
        arrowcolor = color;                     % -- default color
    end
    
    if nargin > 7                               % If no 8th argument:
        arrowwidth = width;                     % -- default width
        
        if width < .1                           % If width too small              
            arrowwidth = .01;                   % -- set to minimum width
            detail = 500;                       % -- set high detail
        end
    end
                                                % Start buffer data 
    X = [x0, x1, nan];                          % (tail --> head)
    Y = [y0, y1, nan];                          % 'nan' separates lines
    Z = [z0, z1, nan];
    
    u = [x1 - x0; y1 - y0; z1 - z0];            % Directional vector
    uN = norm(u);                               % Length
    
    for i = 1:detail                            % Loop: Cone buffer
        random_vec = ones(3,1)-2*rand(3,1);     % -- Generate random vector
        rad = cross(u, random_vec);             % -- Cross product
        rad = rad/norm(rad)*uN/30;              % -- Normalize radius
        X = [X, x1, x1 - (x1 - x0)/7 + rad(1), nan]; % -- Draw cone-line
        Y = [Y, y1, y1 - (y1 - y0)/7 + rad(2), nan];
        Z = [Z, z1, z1 - (z1 - z0)/7 + rad(3), nan];
    end
    
    Arrow = line(X, Y, Z, ...                   % Output 'Arrow'
             'linewidth', arrowwidth, ...
                 'color', arrowcolor); 
end